<?php

/* AccessBundle.php */

/**
 * Name: AccessBundle.php [PHP class definition]
 * Author: R. Joseph Newton
 * Class Provided: AccessBundle
 * Purpose: Provides a AccessBundle class for use in
 *  project management applications.
 * Last Modified: March 30, 2007
 */

include_once('MySQLAdapterProjectManager.php');
include_once('ProjectPrivilege.php');
include_once('ProjectSpacePrivilege.php');

// Scope types
define('PROJECT_SPACE', 0);
define('PROJECT'      , 1);

// Simple permissions
define('RECORD_PROGRESS',        2);
define('VIEW',                   4);
define('DECLARE_MILESTONES',     8);
define('SPECIFY_DELIVERABLES',   16);
define('RECORD_ALLOCATIONS,      32);
define('RECORD_EXPENDITURES,     64);
define('ADD_PERSON',             128);
define('DELEGATE_PRIVILEGES,     256);

// Compouund permissions
define('TRACK_PROJECT',          6);
define('MANAGE_PROGRESS',        30);
define('MANAGE_ACCOUNTS',        96);
define('MANAGE_PROJECT',         254);
define('SUPERVISE',              510);
define('ADMINISTER',             388);

class AccessBundle {
   private _db_adapter;
   private _project_space_tree;
   private _person_guid;
   private _roles;
   private _privileges;
   
   public function AccessBundle($person_id) {
      $this->_db_adapter = new MySQLAdapterProjectManager();
      
      $this->_person_guid = $person_id;
      $this->_roles = array();
      $this->_privileges = array();
      $this->_privileges['project_spaces'] = array();
      $this->_privileges['projects'] = array();
      $this->_build_project_management_tree();
      $this->_init_privileges();
   }
   
   private function _build_project_management_tree() {
      $this->_project_space_tree = array();
      $this->_load_project_space_tree();
      $this->load_projects_to_tree();
   }
   
   private function _load_project_space_tree() {
      $adapter = $this->_db_adapter;
      
      $query = 'select project_space_guid, parent_space_guid from project_space' .
      ' order by parent_space_guid, project_space_guid';
      $result = $adapter->db_get_all($query);
      foreach($result as $ps_pair) {
         $psid = 'psid' . $ps_pair['project_space_guid'];
         $this->_project_space_tree[$psid] =
          array( 'space'  => $ps_pair['project_space_guid'],
                'parent'  => $ps_pair['parent_space_guid'],
               'projects' => array() );
      }
   }
   
   private function _init_privileges() {
      $adapter = $this->_db_adapter;
      
      $this->_load_roles();
      $this->_attach_role_based_privileges();
      $this->_attach_project_space_privileges();
      $this->_attach_project_privileges();
   }
   
   private function _load_roles() {
      $adapter = $this->_db_adapter;
      
      $person_id = $this->_person_guid;
      $query = 'select participant.role_guid, participant.domain_guid, role.scope' .
      ' from participant join role on participant.role_guid = role.role_guid' .
      " where person_guid = $person_id";
      $result = $adapter->db_get_all($query);
      $roles = array();
      foreach ($result as $role) {
         $this->_roles[] = array('domain'  => $role['domain_guid'],
                            'role'  => $role['role_guid'],
                            'scope' => $role['scope'] );
      }
   }
   
   private function _attach_role_based_privileges() {
      $adapter = $this->_db_adapter;
      
      $this->_attach_space_privileges_to_role();
      $this->_attach_project_privileges_to_role();
   }
   
   private function _attach_space_privileges_to_role() {
   }
}

?>

