<?php

/* AccessRight.php */

/**
 * Name: AccessRight.php [PHP class definition]
 * Author: R. Joseph Newton
 * Class Provided: AccessRight
 * Purpose: Provides a AccessRight class for use in
 *  project management applications.
 * Last Modified: March 30, 2007
 */

include_once('MySQLAdapterProjectManager.php');
include_once('ProjectPrivilege.php');
include_once('ProjectSpacePrivilege.php');

define('PROJECT_SPACE', 0);
define('PROJECT'      , 1);

class AccessRight {
   private $_domain_type;
   private $_domain_guid;
   private $_privilege_guid;
   private $_privilege_name;
   private $_description;
   private $_holder_id;
   private $_is_role;
   private $_subtree_included;
   
   public function AccessRight ($domain_type, $domain_id, $privilege_id, Privilege $privilege,
    $holder_id, $is_role, $subtree_included) {
      $this->_domain_type = $domain_type;
      $this->_domain_guid = $domain_id;
      $this->_privilege_guid = $privilege_id;
      $this->_privilege_name = $privilege->get_privilege_name();
      $this->_description = $privilege->get_description();
      $this->_holder_id = $holder_id;
      $this->_is_role = $is_role;
      $this->_subtree_included = $subtree_included;
   }
   
   public function from_project_space_privilege (ProjectSpacePrivilege $ps_privilege) {
      $domain_type = PROJECT_SPACE;
      $domain_id = $ps_privilege->get_project_space_id();
      $privilege_id = $ps_privilege->get_privilege_id();
      $privilege = $ps_privilege->get_privilege();
      $holder_id = $ps_privilege->get_holder_id();
      $is_role = $ps_privilege->is_role_bound();
      $subtree_included = $ps_privilege->includes_subtree();
      $access_right = new AccessRight($domain_type, $domain_id, $privilege_id, $privilege,
       $holder_id, $is_role, $subtree_included);
      return $access_right;
   }
   
   public function from_project_privilege (ProjectSpacePrivilege $project_privilege) {
      $domain_type = PROJECT;
      $domain_id = $project_privilege->get_project_id();
      $privilege_id = $project_privilege->get_privilege_id();
      $privilege = $project_privilege->get_privilege();
      $holder_id = $project_privilege->get_holder_id();
      $is_role = $project_privilege->is_role_bound();
      $subtree_included = $project_privilege->includes_subtree();
      $access_right = new AccessRight($domain_type, $domain_id, $privilege_id,
       $privilege, $holder_id, $is_role, $subtree_included);
      return $access_right;
   }
}

?>

