<?php
 /**
  * Name: Allocation.php
  * Author: R. Joseph Newton
  * Purpose: Provides Allocation class for Project Manager applications
  * Class provided: Allocation
  * Last modified: February 2, 2007
  */

include_once('project_defs.php');
include_once('MySQLAdapterProjectManager.php');

class Allocation {
   private $_allocation_guid;
   private $_project_guid;
   private $_amount;
   private $_source;
   private $_date_allocated;
   
   private $_title;
   private $_purpose;
   private $_description;
   private $_registered;
   private $_db_adapter;
   private $_modified;
   
   public function Allocation($project_id, $amount, $source, $date_allocated, $title, $purpose,
    $description, $registered) {
      $this->_project_guid = $project_id;
      $this->_amount = $amount;
      $this->_date_allocated = $date_allocated;
      $this->_source = $source;
      $this->_title = $title;
      $this->_purpose = $purpose;
      $this->_description = $description;
      $this->_registered = $registered;
      $this->_db_adapter = new MySQLAdapterProjectManager();
      $this->_modified = true;
   }
   
   public function load_from_array(array $array) {
      $allocation = new Allocation($array['project_guid'], $array['amount'],
       $array['source'], $array['date_allocated'], $array['title'],
       $array['purpose'], $array['description'], $array['registered']);
      $allocation->_allocation_guid = $array['allocation_guid'];
      $allocation->_modified = false;
      return $allocation;
   }
   
   public function get_by_allocation_id($allocation_id) {
      $adapter = new MySQLAdapterProjectManager();
      $allocation = NULL;
      $query = "select * from allocation where allocation_guid = $allocation_id";
      $result = $adapter->db_get_first($query);
      $date_allocated = date_ISO_to_array($result['date_allocated']);
      $description = isset($result['description']) ? $result['description'] : '';
      $allocation = new Allocation($result['project_guid'], $result['amount'], $result['source'],
       $date_allocated, $result['title'], $result['purpose'], $description, $result['registered']);
      $allocation->_allocation_guid = $allocation_id;
      $allocation->_modified = false;
      return $allocation;
   }
   
   public function save() {
      if ( isset($this->_allocation_guid) ) {
         $result = $this->_update();
      } else {
         $result = $this->_create_new();
      }
      if ($result == 1) {
         $this->__modified = false;
      }
      return $result;
   }
   
   private function _update() {
      $adapter = $this->_db_adapter;
      
      $this->_amend();
      
      $source = $adapter->db_escape($this->_source);
      $title = $adapter->db_escape($this->_title);
      $purpose = $adapter->db_escape($this->_purpose);
      $date = date_array_to_ISO($this->_date_allocated);
      $description = $adapter->db_escape($this->_description);
      $command = "update allocation set" .
      " project_guid = $this->_project_guid," .
      " amount = $this->_amount," .
      " source = '$source'," .
      " date_allocated = $date," .
      " title = '$title'," .
      " purpose = '$purpose'," .
      " description = '$description'," .
      " registered = '$this->_registered'" .
      "    where allocation_guid = $this->_allocation_guid";
      $result = $adapter->db_command($command);
      return $result;
   }
   
   private function _amend() {
      $adapter = $this->_db_adapter;
      
      $change_guid = $adapter->create_table_guid('allocation_change');
      $date_now = date('Y-m-d');
      $command = "insert into allocation_change select * from" .
      " (select $change_guid, '$date_now') as change_data join" .
      " (select * from allocation" .
      " where allocation_guid = $this->_allocation_guid) as existing_row;";
      $result = $adapter->db_command($command);
   }
   
   private function _create_new() {
      $adapter = $this->_db_adapter;
      $guid = $adapter->create_table_guid('allocation');
      if (!$guid) {
         return false;
      }
      $this->_allocation_guid = $guid;
      $query = $this->_insert_allocation_query();
      $result = $adapter->db_command($query);
      return $result;
   }
   
   private function _insert_allocation_query() {
      $adapter = $this->_db_adapter;
      
      $source = $adapter->db_escape($this->_source);
      $date_array = $this->_date_allocated;
      $date_allocated = date_array_to_ISO($this->_date_allocated);
      $title = $adapter->db_escape($this->_title);
      $purpose = $adapter->db_escape($this->_purpose);
      $description = $this->_description ? $adapter->db_escape($this->_description) : '';
      
      $query = "insert into allocation values (" .
      "$this->_allocation_guid," .
      " $this->_project_guid," .
      " $this->_amount," .
      " '$source'," .
      " $date_allocated," .
      " '$title'," .
      " '$purpose'," .
      " '$description'," .
      " '$this->_registered')";
      
      return $query;
   }
   
   public function get_allocation_id() {
      return $this->_allocation_guid;
   }
   
   public function get_allocation_project_id() {
      return $this->_project_guid;
   }

   public function reset_allocation_project_id($new_project_id) {
      $this->_project_guid = $new_project_id;
      $this->_modified = true;
   }
   
   public function get_allocation_amount() {
      return $this->_amount;
   }
   
   public function adjust_allocation_amount($adjusted_amount) {
      $this->_amount = $adjusted_amount;
      $this->_modified = true;
   }
   
   public function get_funding_source() {
      return $this->_source;
   }
   
   public function amend_funding_source_designation($amended_designation) {
      $amendment_date = strftime('%B %d %Y %H:%M:%S');
      $this->_source = $amended_designation;
      $this->_modified = true;
   }
   
   public function get_date_allocated() {
      return $this->_date_allocated;
   }
   
   public function amend_date_allocated(array $amended_date) {
      $this->_date_allocated = $amended_date;
      $this->_modified = true;
   }
   
   public function get_line_item_title() {
      return $this->_title;
   }

   public function amend_line_item_title($amended_title) {
      $this->_title = $amended_title;
      $this->_modified = true;
   }
   
   public function get_allocation_purpose() {
      return $this->_purpose;
   }
   
   public function amend_allocation_purpose($amended_purpose) {
      $this->_purpose = $amended_purpose;
      $this->_modified = true;
   }
   
   public function get_allocation_description() {
      return $this->_description;
   }
   
   public function modify_allocation_description($amended_description) {
      $this->_description = $amended_description;
      $this->_modified = true;
   }
   
   public function get_original_registry_date() {
      return $this->_registered;
   }
   
   public function is_modified() {
      return $this->_modified;
   }
   
   public function as_array() {
      $allocation_array = array('allocation_id' => $this->_allocation_guid,
       'project_id' => $this->_project_guid, 'amount' => $this->_amount,
       'source' => $this->_source, 'title' => $this->_title, 'purpose' => $this->_purpose);
      if ( isset($this->_description) ) {
         $allocation_array[] = array('description' => $this->_description);
      }
      return $allocation_array;
   }
   
   public function to_string() {
      $allocation_string = "$this->_allocation_guid: $this->_title\n" .
       "Funding source: $this->_source\n" .
       "Amount                      $this->_amount\n" .
       "Purpose:\n$this->_purpose\n" .
       "Project ID: $this->_project_guid\n";
      if ( isset($this->_description) ) {
         $allocation_string .= "Description:\n $this->_description\n";
      }
      return $allocation_string;
   }
   
   public function to_HTML_string() {
      $allocation_string = "$this->_allocation_guid: $this->_title <br />\n" .
       "Funding source: $this->_source <br />\n" .
       "Amount                      $this->_amount <br />\n" .
       "Purpose:\n$this->_purpose <br />\n" .
       "Project ID: $this->_project_guid <br />\n";
      if ( isset($this->_description) ) {
         $allocation_string .= "Description: <br />\n $this->_description <br />\n";
      }
      return $allocation_string;
   }
}

?>
