<?php
 /**
  * Name: Deliverable.php
  * Author: R. Joseph Newton
  * Purpose: Provides Deliverable class for Project Manager applications
  * Class provided: Deliverable
  * Last modified: February 3, 2007
  */

include_once('project_defs.php');
include_once('MySQLAdapterProjectManager.php');

class Deliverable {
   private $_db_adapter;
   private $_deliverable_guid;
   private $_project_guid;
   private $_title;
   private $_purpose;
   private $_specification;
   private $_date_due;
   private $_date_delivered;
   private $_registered;
   private $_modified;
   
   public function Deliverable($project_guid, $title, $purpose, $specification, array $date_due,
    array $date_delivered = NULL) {
      $this->_project_guid = $project_guid;
      $this->_title = $title;
      $this->_purpose = $purpose;
      $this->_specification = $specification;
      $this->_date_due = $date_due;
      $this->_date_delivered = $date_delivered;
      
      $this->_db_adapter = new MySQLAdapterProjectManager();
      $this->_modified = true;
   }
   
   public function load_from_array(array $array) {
      $date_due = date_ISO_to_array($array['date_due']);
      $date_delivered = date_ISO_to_array($array['date_delivered']);
      $deliverable = new Deliverable($array['project_guid'], $array['title'],
       $array['purpose'], $array['specification'], $date_due,
       $date_delivered);
      $deliverable->_deliverable_guid = $array['deliverable_guid'];
      $deliverable->_modified = false;
      return $deliverable;
   }
   
   public function get_by_deliverable_id($deliverable_id) {
      $adapter = new MySQLAdapterProjectManager();
      
      $deliverable = NULL;
/*      print "deliverable_id is seen as $deliverable_id<br>\n"; */
      $query = "select * from deliverable where deliverable_guid = $deliverable_id";
      $result = $adapter->db_get_first($query);
      $date_due = date_ISO_to_array($result['date_due']);
      $date_delivered = date_ISO_to_array($result['date_delivered']);
      $deliverable = new Deliverable($result['project_guid'], $result['title'],
       $result['purpose'], $result['specification'], $date_due, $date_delivered,
       $result['registered']);
      $deliverable->_deliverable_guid = $deliverable_id;
      $deliverable->_modified = false;
      return $deliverable;
   }
   
   public function save() {
      if ( isset($this->_deliverable_guid) ) {
         $result = $this->_update();
      } else {
         $result = $this->_create_new();
      }
      if ($result == 1) {
         $this->__modified = false;
      }
   }
   
   private function _update() {
      $adapter = $this->_db_adapter;
      
      $this->_amend();
      
      $title = $adapter->db_escape($this->_title);
      $purpose = $adapter->db_escape($this->_purpose);
      $specification = $adapter->db_escape($this->_specification);
      $date_due = date_array_to_ISO($this->_date_due);
      $date_delivered = date_array_to_ISO($this->_date_delivered);
      $command = "update deliverable set project_guid = $this->_project_guid," .
      "title = '$title', purpose = '$purpose', specification = '$specification'," .
      " date_due = $date_due, date_delivered = $date_delivered";
      $command .= " where deliverable_guid = $this->_deliverable_guid";
      $result = $adapter->db_command($command);
      return $result;
   }
   
   private function _amend() {
      $adapter = $this->_db_adapter;
      
      $change_guid = $adapter->create_table_guid('deliverable_change');
      $date_now = date('Y-m-d');
      $command = "insert into deliverable_change select * from" .
      " (select $change_guid, '$date_now') as change_data join" .
      " (select * from deliverable" .
      " where deliverable_guid = $this->_deliverable_guid) as existing_row;";
      $result = $adapter->db_command($command);
   }
   
   private function _create_new() {
      $adapter = $this->_db_adapter;
      
      $guid = $adapter->create_table_guid('deliverable');
      if (!$guid) {
         return false;
      }
      $this->_deliverable_guid = $guid;
      $this->_registered = Date('Y-m-d');
      $command = $this->_insert_deliverable_query();
      $result = $adapter->db_command($command);
      return $result;
   }
   
   private function _insert_deliverable_query() {
      $adapter = $this->_db_adapter;
      
      $title = $adapter->db_escape($this->_title);
      $purpose = $adapter->db_escape($this->_purpose);
      $specification = $adapter->db_escape($this->_specification);
      $date_due = isset($this->_date_due['year']) ?
       date_array_to_ISO($this->_date_due) : NULL;
      $date_delivered = isset($this->_date_delivered['year']) ?
       date_array_to_ISO($this->_date_delivered) : NULL;
      $query = "insert into deliverable values ($this->_deliverable_guid," .
      " $this->_project_guid, '$title', '$purpose', '$specification', $date_due," .
      " $date_delivered, '$this->_registered')";
      return $query;
   }

   public function get_deliverable_id() {
      return $this->_deliverable_guid;
   }
   
   public function get_project_id() {
      return $this->_project_guid;
   }
   
   public function change_project_designation($new_project_id) {
      $this->_project_guid = $new_project_id;
      $this->_modified = true;
   }
   
   public function get_title() {
      return $this->_title;
   }
   
   public function change_title($new_title) {
      $this->_title = $new_title;
      $this->_modified = true;
   }
   
   public function get_purpose() {
      return $this->_purpose;
   }
   
   public function change_purpose($new_purpose) {
      $this->_purpose = $new_purpose;
      $this->_modified = true;
   }
   
   public function get_specification() {
      return $this->_specification;
   }
   
   public function amend_specification($new_specification) {
      $this->_specification = $new_specification;
      $this->_modified = true;
   }
   
   public function get_date_due() {
      return $this->_date_due;
   }
   
   public function amend_date_due($new_due_date) {
      $this->_date_due = $new_due_date;
      $this->_modified = true;
   }
   
   public function get_date_delivered() {
      return $this->_date_delivered;
   }
   
   public function record_date_delivered($delivery_date) {
      $this->_date_delivered = $delivery_date;
      $this->_modified = true;
   }

   public function get_original_registry_date() {
      return $this->_registered;
   }
   
   public function as_array() {
      $deliverable_array = array('deliverable_id' => $this->_deliverable_guid,
       'project_id' => $this->_project_guid, 'title' => $this->_title,
       'purpose' => $this->_purpose, 'specification' => $this->_specification);
      if ( isset($this->_date_due['year']) ) {
         $deliverable_array[] = array('date_due' => $this->_date_due);
      }
      if ( isset($this->_date_delivered['year']) ) {
         $deliverable_array[] = array('date_delivered' => $this->_date_delivered);
      }
      return $deliverable_array;
   }
   
   public function to_string() {
      $date_goal = date_array_to_ISO($this->_date_goal);
      $date_delivered = date_array_to_ISO($this->_date_delivered);
      $deliverable_string = "$this->_deliverable_guid: $this->_title\n" .
       "Project ID: $this->_project_guid\n" .
       "Purpose: $this->_purpose\n";
      if ( isset($date_due) ) {
         $deliverable_string .= "Date due: $date_due\n";
      }
      if ( isset($this->_specification) ) {
         $deliverable_string .= "Specification:\n$this->_specification\n";
      }
      if ( isset($date_delivered) ) {
         $deliverable_string .= "Date delivered: $date_delivered\n";
      }
      return $deliverable_string;
   }
}

?>
