<?php
/* Privilege.php */

/**
 * Name: Privilege.php [PHP class definition]
 * Author: R. Joseph Newton
 * Class Provided: Privilege
 * Purpose: Provides a Privilege class for use in project management applications
 * Last Modified: March 29, 2007
 */

include_once('MySQLAdapterProjectManager.php');

/* Preset privileges: */
define('MANAGE', 1);
define('VIEW', 2);
define('RECORD_PROGRESS', 3);
define('RECORD_EXPENDITIURES', 4);
define('RECORD_ALLOCATIONS', 5);
define('MANAGE_ACCOUNTS', 6);
define('ADD_PARTICIPANTS', 7);
define('DELEGATE_PRIVILEGES', 8);

class Privilege {
   private $_db_adapter;
   private $_privilege_guid;
   private $_privilege_name;
   private $_description;
   private $_modified;
   
   public function Privilege ($privilege_name, $description) {
      $this->db_adapter = new MySQLAdapterProjectManager();
      
      $this->_privilege_name = $privilege_name;
      $this->_description = $description;
      $this->_modified = true;
   }
   
   public function save() {
      if ( isset($this->privilege_guid) ) {
         $this->_update();
      } else {
         $this->_create_new();
      }
   }
   
   private function _update() {
      $adapter = $this->db_adapter;
      
      $description = $adapter->db_escape($this->_description);
      $command = 'update privilege set description = ' . $description .
      ' where privilege_guid = ' . $this->_privilege_guid;
      $result = $db_command($command);
      $this->_modified = false;
      return $result;
   }
   
   private function _create_new() {
      $adapter = $this->db_adapter;
      
      $this->_privilege_guid = $adapter->create_table_guid('privilege');
      $privilege_name = $adapter->db_escape($this->_privilege_name);
      $description = $adapter->db_escape($this->_description);
      $command = 'insert into privilege (privilege_guid, privilege_name, description)' .
      " values ($this->_privilege_guid, $privilege_name, $description)";
      $result = db_command($command);
      $this->_modified = false;
      return $result;
   }
   
   public function get_by_privilege_id($privilege_id) {
      $adapter = new MySQLAdapterProjectManager();
      
      $query = 'select * from privilege where privilege_guid = ' .
       $privilege_id;
      $result = $adapter->db_get_first($query);
      $privilege_name = $result['privilege_name'];
      $description = $result['description'];
      $privilege = new Privilege($privilege_name, $description);
      $privilege->_privilege_guid = $privilege_id;
      $privilege->_modified = false;
      return $privilege;
   }
   
   public function get_privilege_id() {
      return $this->_privilege_guid;
   }
   
   public function get_privilege_name () {
      return $this->_privilege_name;
   }
   
   public function get_description () {
      return $this->_description;
   }
   
   public function modify_description($new_description) {
      $this->_description = $new_description;
   }
}
?>

