<?php
 /**
  * Name: ProjectManager.php
  * Author: R. Joseph Newton
  * Purpose: Manages projects using classes fom Project Manager package
  * Class provided: Project Manager
  */

include_once('project_defs.php');
include_once('utilities.php');
include_once('MySQLAdapterProjectManager.php');
include_once('ProjectSpace.php');
include_once('Project.php');
include_once('Task.php');

$active_process = '';
$page_title = 'Project Manager';
$current_template = 'general.tmpl.php';
$allocation_list = '';
$expenditure_list = '';
global $task_or_project;

/* dump_array_to_page($_POST); */  /* STATIC DEBUG */

class ProjectManager {
   private $_db_adapter;
   
   private $_current_project_space;
   private $_current_project_space_id;
   private $_current_project_id;
   private $_current_project;
   private $_current_allocation;
   private $_current_expenditure;
   private $_current_milestone;
   private $_current_deliverable;
   
   private $_person_ids;
   private $_persons;
   private $_person_onboard_ids;
   private $_persons_onboard;
   private $_persons_to_edit;
   private $_projects;
   private $_tasks;
   
   public function ProjectManager() {
      $this->_current_project_space_id = isset($_POST['selected_project_space_id']) ? $_POST['selected_project_space_id'] :
       ( isset($_POST['current_project_space_id']) ? $_POST['current_project_space_id'] : 0);
      $this->_current_project_id =
       (isset($_POST['action']) && $_POST['action'] == 'Select project' && isset( $_POST['selected_project_id'] ) ) ?
       $_POST['selected_project_id'] :
        (isset($_POST['current_project_id']) ? $_POST['current_project_id'] : NULL) /* ) */;
      $this->_db_adapter = new MySQLAdapterProjectManager();
      $this->_person_onboard_ids = array();
      $this->_persons_onboard = array();
      $this->_persons = array();
      $this->_projects = array();
      $this->_tasks = array();
      $this->_init();
   }
   
   private function _init() {
      global $task_or_project;
      $project_id = $this->_current_project_id;
      $task = Task::get_by_task_id($project_id);
      $task_or_project = isset($task) ? 'Task' : 'Project';
      $this->_set_active_process();
      $projects = $this->_load_projects();
      if ( isset($project_id) ) {
         $this->_load_people_by_current_project();
      } else {
         $this->_load_people_by_current_project_space();
      }
   }
   
   private function _set_active_process() {
      $action = isset($_POST['action']) ? $_POST['action'] : NULL;
      if ( !isset($action) ) {
         return;
      }
      global $current_template;
      global $active_process;
      global $task_or_project;
      $this->_set_active_project();
      if (strpos($action, 'AJAX', 0) === 0) {
         $this->start_ajax_server($action);
      } elseif ( strpos($action, 'project', 4) && strpos($action, 'project', 4) < 13 ) {
         $this->_switch_action_on_project($action);
      } else {
 /*        if ( !isset($this->_current_project) ) {
            return;
         }             */
         $this->_switch_action_for_current_project($action);
      }
      if ( isset($this->_current_project) ) {
         $this->_current_project->load_containers();
      }
   }
   
   private function _set_active_project() {
      if ( !isset($this->_current_project_id) ) {
         return;
      }
      $adapter = $this->_db_adapter;
      $guid = $this->_current_project_id;
      $this->_current_project = Project::get_by_project_id($guid);
   }
   
   private function start_ajax_server($action) {
      switch ($action) {
         case 'AJAX create person':
            $this->_ajax_create_person();
         case 'AJAX reload available persons':
            $this->_ajax_reload_persons_available();
      }
   }
   
   private function _ajax_create_person() {
      $given = $_POST['given'];
      $last = $_POST['last'];
      $first = $_POST['first'];
      $middle = $_POST['middle'];
      $contact_mode = $_POST['contact_mode'];
      $person = new Person($given, $last, $contact_mode, $first, $middle);
      $result_person = $person->save();
      $person_id = $person->get_person_id();
      header('Content-type: text/plain');
      $project_space = $_POST['project_space'];
      $result_association = $this->_make_person_available($person_id, $project_space);
      $person_insert_status = $result_person && $result_association ? 'succeeded' : 'failed';
      exit($person_insert_status);
   }
   
   private function _make_person_available($person_id, $project_space) {
      $adapter = $this->_db_adapter;
      
      $query = "insert into person_visible values ($person_id, $project_space)";
      $result = $adapter->db_command($query);
      return $result;
   }
   
   private function _ajax_reload_persons_available() {
      if (isset($_POST['current_project_id'])) {
         $this->_load_people_available_but_not_on_current_project($_POST['current_project_id']);
      } else {
         $this->_load_people_by_current_project_space();
      }
      $select_innards = $this->_assemble_persons_available_options_html();
      exit($select_innards);
   }
   
   private function _assemble_persons_available_options_html() {
      $select_innards = "__PERSONS_AVAILABLE__\n";
      
      $people = $this->get_persons();
      foreach ($people as $person) {
         $person_id = $person['person_guid'];
         $person_name = $person['name_given'] . ' ' . $person['name_last'];
         $select_innards .= "               ";
         $select_innards .= "<option value=\"$person_id\"> $person_name </option>\n";
      }
      return $select_innards;
  }
   
   private function _switch_action_on_project($action) {
      global $task_or_project;
      switch ($action) {
         case 'Select project':
            $active_process = "manage $task_or_project";
            break;
         case 'New project' :
            $this->_add_project();
            break;
         case 'commit new project':
            global $active_process;
            $active_process = 'commit new project';
            $this->_commit_new_project();
            break;
      }
   }
   
   private function _add_project() {
      global $current_template;
      global $active_process;
      
      $this->_load_people_by_current_project_space();
      $active_process = 'add project';
      $current_template = 'project_entry.tmpl.php';
   }
   
   private function _commit_new_project() {
      $adapter = $this->_db_adapter;
      
      list ($name, $description, $lead, $co_lead, $start_date, $projected_finish,
       $finish_date) = $this->_get_new_project_arguments($adapter);
      $project_space_id = $this->_current_project_space_id;
      $project_space = ProjectSpace::get_by_project_space_id($project_space_id);
      $project = $project_space->add_project($name, $description,
       $lead, $co_lead, $start_date, $projected_finish, $finish_date);
      $this->_current_project_id = $project->get_project_id();
      $this->_current_project = $project;
      return $this->_current_project_id;
   }

   private function _get_new_project_arguments(MySQLAdapterProjectManager $adapter) {
      $alloc_array = extract_from_POST_vars('project-');
      $name = $adapter->db_escape($alloc_array['name']);
      $description = $adapter->db_escape($alloc_array['description']);
      $lead = $alloc_array['lead'];
      $co_lead = isset($alloc_array['co_lead']) ? $alloc_array['co_lead'] : NULL;
      $start_date = array('year' => $alloc_array['start_date-year'],
        'month' => $alloc_array['start_date-month'],
        'day' => $alloc_array['start_date-day']);
      $projected_finish = isset($alloc_aray['projected_finish-year']) ?
       array('year' => $alloc_array['projected_finish-year'],
        'month' => $alloc_array['projected_finish-month'],
        'day' => $alloc_array['projected_finish-day'])  : array();
      $finish_date = isset($alloc_array['finish_date-year']) ?
       array('year' => $alloc_array['finish_date-year'],
        'month' => $alloc_array['finish_date-month'],
        'day' => $alloc_array['finish_date-day'])  : array();
      return array($name, $description, $lead, $co_lead, $start_date, $projected_finish, $finish_date);
   }

   private function _switch_action_for_current_project($action) {
      if ( strpos($action, 'task', 4) ) {
         $this->_switch_action_on_task($action);
      } elseif ( strpos($action, 'person', 4) ) {
         $this->_switch_action_on_person($action);
      } elseif ( strpos($action, 'allocation', 4) ) {
         $this->_switch_action_on_allocation($action);
      } elseif ( strpos($action, 'expenditure', 4) ) {
         $this->_switch_action_on_expenditure($action);
      } elseif ( strpos($action, 'milestone', 4) ) {
         $this->_switch_action_on_milestone($action);
      } elseif ( strpos($action, 'deliverable', 4) ) {
         $this->_switch_action_on_deliverable($action);
      } else {
      }
   }
   
   private function _switch_action_on_task($action) {
      switch ($action) {
         case 'New task':
            $this->_add_new_task();
            break;
         case 'commit new task':
            $this->_commit_new_task();
            break;
      }
   }
   
   private function _add_new_task() {
      global $current_template;
      global $active_process;
      
      $active_process = 'add task';
      $current_template = 'project_entry.tmpl.php';
   }
   
   private function _commit_new_task() {
      $adapter = $this->_db_adapter;
      global $active_process;
      
      $active_process = 'commit new task';
      list ($name, $description, $lead, $co_lead, $start_date, $projected_finish,
       $finish_date) = $this->_get_new_project_arguments($adapter);
      $project_space_id = $this->_current_project_space_id;
      $project_space = ProjectSpace::get_by_project_space_id($project_space_id);
      $parent_project = $this->_current_project_id;
      $task = $project_space->add_task($parent_project, $name, $description,
       $lead, $co_lead, $start_date, $projected_finish, $finish_date);
      $this->_current_project_id = $task->get_project_id();
      $this->_current_project = $task;
      return $this->_current_project_id;
   }
   
   private function _switch_action_on_person($action) {
      switch ($action) {
       /*  case 'Create new":
            $this->create_new_person(); */
         case 'Add person to project':
            $this->_add_participants_to_project();
            break;
         case 'Remove person from project':
            $this->_remove_participants_from_project();
            break;
         case 'confirm add persons':
            $this->_confirm_add_participants();
      }
   }
   
   private function _add_participants_to_project() {
      if ( !isset($_POST['available_person_ids']) ) {
         return;
      }
      
      global $current_template;
      global $active_process;
      $active_process = 'add participants';
      $current_template = 'add_participant.tmpl.php';
      $adapter = $this->_db_adapter;
      $participant_list = join(', ', $_POST['available_person_ids']);
      $query = "select * from person where person_guid in ($participant_list)" .
      '    order by name_given, name_last';
      $this->_persons_to_edit = $adapter->db_get_all($query);
      $this->_set_active_project();
   }
   
   private function _remove_participants_from_project() {
      if ( !isset($_POST['person_onboard_ids']) ) {
         return;
      }
      
      global $active_process;
      $active_process = 'remove participants';
      $adapter = $this->_db_adapter;
      $project_id = $this->_current_project_id;
      $participant_list = join(', ', $_POST['person_onboard_ids']);
      $command = "delete from participant" .
      " where person_guid in ($participant_list) and domain_guid = $project_id";
      $result = $adapter->db_command($command);
      $this->_set_active_project();
   }
   
   private function _confirm_add_participants() {
      global $active_process;
      $active_process = 'confirm add participants';
      $adapter = $this->_db_adapter;
      
      $project_id = $this->get_current_project_id();
      $base_string = 'is_confirmed_';
      $base_length = strlen($base_string);
      $persons_to_confirm = array();
      $person_ids_to_confirm = array();
      foreach ( $_POST as $post_key => $value ) {
         if (strpos($post_key, $base_string) === 0) {
            if ( $value != 'true' ) {
               continue;
            }
            $person_guid = substr($post_key, $base_length);
            $person_ids_to_confirm[] = $person_guid;
            $role_guid = $_POST['person_role-' . $person_guid];
            $update_information = array();
            $query = "insert into participant values ($person_guid, $project_id, $role_guid)";
            $result = $adapter->db_command($query);
            $update_information['participant_insert_result'] = $result;
            $this->_participant_edit($person_guid, $update_information);
         }
      }
   }
   
   private function _participant_edit($person_guid, array &$update_information) {
      global $active_process;
      $adapter = $this->_db_adapter;
      
      $given_name_key =  'given_name_' . $person_guid . '_edit';
      if ( !isset($_POST[$given_name_key]) ) {
         return;
      }
      $given_name = $_POST[$given_name_key];
      $last_name = isset($_POST['last_name_' . $person_guid . '_edit']) ?
       $_POST['last_name_' . $person_guid . '_edit'] : '';
      $preferred_mode = isset($_POST['contact_preference_' . $person_guid . '_edit']) ?
       $_POST['contact_preference_' . $person_guid . '_edit'] : '';
      $first_name = isset($_POST['first_name_' . $person_guid . '_edit']) ?
       $_POST['first_name_' . $person_guid . '_edit'] : '';
      $middle_name = isset($_POST['middle_name_' . $person_guid . '_edit']) ?
       $_POST['middle_name_' . $person_guid . '_edit'] : '';
      $command = 'update person set' .
      " name_given = '$given_name'," .
      " name_last = '$last_name'," .
      " preferred_mode = '$preferred_mode'," .
      " name_first = '$first_name'," .
      " name_middle = '$middle_name'" .
      " where person_guid = $person_guid";
      $result = $adapter->db_command($command);
      $update_information['participant_edit_result'] = $result;
   }
   
   private function _switch_action_on_allocation($action) {
      switch ($action) {
         case 'add allocation':
            $this->_add_allocation();
            break;
         case 'commit new allocation':
            $this->_commit_allocation();
            break;
         case 'view allocation':
            $this->_view_allocation();
            break;
         case 'edit allocation':
            $this->_edit_allocation();
            break;
         case 'commit amended allocation':
            $this->_commit_allocation_changes();
      }
   }
   
   private function _add_allocation() {
      global $current_template;
      global $active_process;
      $active_process = 'add allocation';
      $current_template = 'allocation_entry.tmpl.php';
   }
   
   private function _commit_allocation() {
      global $active_process;
      $active_process = 'commit allocation';
      $current_project_id = $this->get_current_project_id();
      $registered = date('Y-m-d');
      $alloc_array = extract_from_POST_vars('allocation-');
      $allocation = new Allocation($current_project_id, $alloc_array['amount'],
       $alloc_array['source'], array('year' => $alloc_array['year'], 'month' => $alloc_array['month'],
        'day' => $alloc_array['day']), $alloc_array['title'], $alloc_array['purpose'],
       $alloc_array['description'], $registered);
      $result = $allocation->save();
      return $result;
   }
   
   private function _view_allocation() {
      global $current_template;
      global $active_process;
      global $current_allocation;
      
      $current_allocation_id = $_POST['allocation_id'];
      $current_allocation = Allocation::get_by_allocation_id($current_allocation_id);
      $active_process = 'view allocation';
      $current_template = 'allocation_view.tmpl.php';
   }
   
   private function _edit_allocation() {
      global $current_template;
      global $active_process;
      global $current_allocation;
      
      if ( !isset($_POST['allocation_id'])) {
         return;
      }
      $current_template = 'allocation_edit.tmpl.php';
      $current_allocation_id = $_POST['allocation_id'];
      $current_allocation = Allocation::get_by_allocation_id($current_allocation_id);
      $active_process = 'edit allocation';
   }
   
   private function _commit_allocation_changes() {
      global $active_process;
      $active_process = 'commit allocation changes';
      $allocation_id = $_POST['allocation_id'];
      if ( !isset($allocation_id) ) {
         return;
      }
      $allocation = Allocation::get_by_allocation_id($allocation_id);
      $edited_allocation = extract_from_POST_vars('allocation-');
      $allocation->reset_allocation_project_id( $this->get_current_project_id() );
      $allocation->adjust_allocation_amount($edited_allocation['amount']);
      $allocation->amend_funding_source_designation($edited_allocation['source']);
      $allocation->amend_date_allocated( array('year' =>$edited_allocation['year'],
       'month' => $edited_allocation['month'], 'day' => $edited_allocation['day']) );
      $allocation->amend_line_item_title($edited_allocation['title']);
      $allocation->amend_allocation_purpose($edited_allocation['purpose']);
      $allocation->modify_allocation_description($edited_allocation['description']);
      $allocation->save();
   }
   
   private function _switch_action_on_expenditure($action) {
      switch ($action) {
         case 'view expenditure':
            $this->_view_expenditure();
            break;
         case 'add expenditure':
            $this->_add_expenditure();
            break;
         case 'commit new expenditure':
            $this->_commit_expenditure();
            break;
         case 'amend expenditure':
            $this->_edit_expenditure();
            break;
         case 'commit amended expenditure':
            $this->_commit_amended_expenditure();
      }
   }
   
   private function _view_expenditure() {
      global $current_template;
      global $active_process;
      global $current_expenditure;
      
      $current_expenditure_id = $_POST['expenditure_id'];
      $current_expenditure = Expenditure::get_by_expenditure_id($current_expenditure_id);
      $active_process = 'view expenditure';
      $current_template = 'expenditure_view.tmpl.php';
   }
   
   private function _add_expenditure() {
      global $current_template;
      global $active_process;
      $active_process = 'add expenditure';
      $current_template = 'expenditure_entry.tmpl.php';
   }
   
   private function _commit_expenditure() {
      global $active_process;
      $active_process = 'commit expenditure';
      $current_project_id = $this->get_current_project_id();
      $registered = date('Y-m-d');
      $base_string = 'expenditure-';
      $expend_array = extract_from_POST_vars($base_string);
      $expenditure = new Expenditure($current_project_id, $expend_array['authorized_by'],
       $expend_array['amount'], array(
         'year' => $expend_array['year'],
         'month' => $expend_array['month'],
         'day' => $expend_array['day']),
       $expend_array['payee'], $expend_array['title'],
       $expend_array['description'], $registered);
      $result = $expenditure->save();
      return $result;
   }
   
   private function _edit_expenditure() {
      global $current_template;
      global $active_process;
      global $current_expenditure;
      
      $current_template = 'expenditure_edit.tmpl.php';
      $current_expenditure_id = $_POST['expenditure_id'];
      $current_expenditure = Expenditure::get_by_expenditure_id($current_expenditure_id);
      $active_process = 'edit expenditure';
   }
   
   private function _commit_amended_expenditure() {
      global $active_process;
      $active_process = 'commit expenditure changes';
      $expenditure_id = $_POST['expenditure_id'];
      if ( !isset($expenditure_id) ) {
         return;
      }
      $expenditure = Expenditure::get_by_expenditure_id($expenditure_id);
      $edited_expenditure = extract_from_POST_vars('expenditure-');
      $expenditure->change_project_designation( $this->get_current_project_id() );
      $expenditure->adjust_amount($edited_expenditure['amount']);
      $expenditure->change_payee($edited_expenditure['payee']);
      $expenditure->amend_date_incurred( array('year' => $edited_expenditure['year'],
       'month' => $edited_expenditure['month'], 'day' => $edited_expenditure['day']) );
      $expenditure->amend_line_item_title($edited_expenditure['title']);
      $expenditure->amend_authorized_by($edited_expenditure['authorized_by']);
      $expenditure->modify_expenditure_description($edited_expenditure['description']);
      $expenditure->save();
   }
   
   private function _switch_action_on_milestone($action) {
      switch ($action) {
         case 'view milestone':
            $this->_view_milestone();
            break;
         case 'set milestone':
            $this->_set_new_milestone();
            break;
         case 'commit new milestone':
            $this->_commit_new_milestone();
            break;
         case 'edit milestone':
            $this->_edit_milestone();
            break;
         case 'commit amended milestone':
            $this->_commit_amended_milestone();
      }
   }
   
   private function _view_milestone() {
      global $current_template;
      global $active_process;
      global $current_milestone;
      
      $current_milestone_id = $_POST['milestone_id'];
      $current_milestone = Milestone::get_by_milestone_id($current_milestone_id);
      $active_process = 'view milestone';
      $current_template = 'milestone_view.tmpl.php';
   }
   
   private function _set_new_milestone() {
      global $current_template;
      global $active_process;
      
      $current_template = 'milestone_entry.tmpl.php';
      $active_process = 'set new milestone';
   }
   
   private function _commit_new_milestone() {
      global $active_process;
      
      $active_process = 'commit new milestone';
      $current_project_id = $this->get_current_project_id();
      $registered = date('Y-m-d');
      $base_string = 'milestone-';
      $milestone_array = extract_from_POST_vars($base_string);
      $date_target =  isset($milestone_array['target_year']) ?
       array('year' => $milestone_array['target-year'],
            'month' => $milestone_array['target-month'],
            'day' => $milestone_array['target-day']) : NULL;
      $date_achieved =  isset($milestone_array['achieved_year']) ?
       array('year' => $milestone_array['achieved-year'],
            'month' => $milestone_array['achieved-month'],
            'day' => $milestone_array['achieved-day']) : NULL;
      $registered = date('Y-m-d');
      $milestone = new Milestone($current_project_id, $milestone_array['title'],
       $milestone_array['significance'], $milestone_array['description'],
       $date_target, $date_achieved, $registered);
      $result = $milestone->save();
      return $result;
   }
   
   private function _edit_milestone() {
      global $current_template;
      global $active_process;
      global $current_milestone;
      
      $current_template = 'milestone_edit.tmpl.php';
      $current_milestone_id = $_POST['milestone_id'];
      $current_milestone = Milestone::get_by_milestone_id($current_milestone_id);
      $active_process = 'edit milestone';
   }
   
   private function _commit_amended_milestone() {
      global $active_process;
      
      $active_process = 'commit milestone changes';
      $milestone_id = $_POST['milestone_id'];
      if ( !isset($milestone_id) ) {
         return;
      }
      $milestone = Milestone::get_by_milestone_id($milestone_id);
      $edited_milestone = extract_from_POST_vars('milestone-');
      $milestone->change_project_designation( $this->get_current_project_id() );
      $milestone->change_title($edited_milestone['title']);
      $milestone->adjust_target_date( array('year' => $edited_milestone['target-year'],
       'month' => $edited_milestone['target-month'], 'day' => $edited_milestone['target-day']) );
      $milestone->record_date_achieved( array('year' => $edited_milestone['achieved-year'],
       'month' => $edited_milestone['achieved-month'], 'day' => $edited_milestone['achieved-day']) );
      $milestone->re_express_significance($edited_milestone['significance']);
      $milestone->modify_milestone_description($edited_milestone['description']);
      $milestone->save();
   }
   
   private function _switch_action_on_deliverable($action) {
      switch ($action) {
         case 'view deliverable':
            $this->_view_deliverable();
            break;
         case 'specify deliverable':
            $this->_specify_new_deliverable();
            break;
         case 'commit new deliverable':
            $this->_commit_new_deliverable();
            break;
         case 'edit deliverable':
            $this->_edit_deliverable();
            break;
         case 'commit amended deliverable':
            $this->_commit_amended_deliverable();
            break;
      }
   }
   
   private function _view_deliverable() {
      global $current_template;
      global $active_process;
      global $current_deliverable;
      
      $current_deliverable_id = $_POST['deliverable_id'];
      $current_deliverable = deliverable::get_by_deliverable_id($current_deliverable_id);
      $active_process = 'view deliverable';
      $current_template = 'deliverable_view.tmpl.php';
   }
   
   private function _specify_new_deliverable() {
      global $current_template;
      global $active_process;
      
      $current_template = 'deliverable_entry.tmpl.php';
      $active_process = 'specify new deliverable';
   }
   
   private function _commit_new_deliverable() {
      global $active_process;
      
      $active_process = 'commit new deliverable';
      $current_project_id = $this->get_current_project_id();
      $registered = date('Y-m-d');
      $base_string = 'deliverable-';
      $deliverable_array = extract_from_POST_vars($base_string);
      $date_due =  isset($deliverable_array['due-year']) ?
       array('year' => $deliverable_array['due-year'],
            'month' => $deliverable_array['due-month'],
            'day' => $deliverable_array['due-day']) : array();
      $date_delivered =  isset($deliverable_array['delivered-year']) ?
       array('year' => $deliverable_array['delivered-year'],
            'month' => $deliverable_array['delivered-month'],
            'day' => $deliverable_array['delivered-day']) : array();
      $registered = date('Y-m-d');
      $deliverable = new Deliverable($current_project_id, $deliverable_array['title'],
       $deliverable_array['purpose'], $deliverable_array['specification'],
       $date_due, $date_delivered, $registered);
      $result = $deliverable->save();
      return $result;
   }
   
   private function _edit_deliverable() {
      global $current_template;
      global $active_process;
      global $current_deliverable;
      
      $current_template = 'deliverable_edit.tmpl.php';
      $current_deliverable_id = $_POST['deliverable_id'];
      $current_deliverable = Deliverable::get_by_deliverable_id($current_deliverable_id);
      $active_process = 'edit deliverable';
   }
   
   private function _commit_amended_deliverable() {
      global $active_process;
      
      $active_process = 'commit deliverable changes';
      $deliverable_id = $_POST['deliverable_id'];
      if ( !isset($deliverable_id) ) {
         return;
      }
      $deliverable = Deliverable::get_by_deliverable_id($deliverable_id);
      $edited_deliverable = extract_from_POST_vars('deliverable-');
      $deliverable->change_project_designation( $this->get_current_project_id() );
      $deliverable->change_title($edited_deliverable['title']);
      $deliverable->amend_date_due( array('year' => $edited_deliverable['due-year'],
       'month' => $edited_deliverable['due-month'], 'day' => $edited_deliverable['due-day']) );
      $deliverable->record_date_delivered( array('year' => $edited_deliverable['delivered-year'],
       'month' => $edited_deliverable['delivered-month'], 'day' => $edited_deliverable['delivered-day']) );
      $deliverable->change_purpose($edited_deliverable['purpose']);
      $deliverable->amend_specification($edited_deliverable['specification']);
      $deliverable->save();
   }
   
   private function _load_projects() {
      if ( isset($this->_current_project_id) ) {
         $this->_load_child_tasks();
      } else {
         $this->_load_root_projects();
      }
   }
  
   private function _load_child_tasks() {
      $adapter = $this->_db_adapter;
      $guid = $this->_current_project_id;
      $query = 'select * from project left outer join task' .
               ' on project.project_guid = task.task_guid' .
               "    where task.parent_project = $guid";
      $child_projects = $adapter->db_get_all($query);
      $this->_tasks['guids'] = array();
      foreach ($child_projects as $project) {
         $guid = $project['project_guid'];
         $this->_tasks[$guid] = $project;
         $name = $project['name'];
         $this->_tasks['guids'][$name] = $guid;
      }
   }
   
   private function _load_root_projects() {
      $adapter = $this->_db_adapter;
      $query = 'select * from project left outer join task' .
               '  on project.project_guid = task.task_guid' .
               '     where task.task_guid is null' .
               '     and project_space_guid = ' . $this->_current_project_space_id;
      $root_projects = $adapter->db_get_all($query);
      $this->_projects['guids'] = array();
      foreach ($root_projects as $project) {
         $guid = $project['project_guid'];
         $this->_projects[$guid] = $project;
         $name = $project['name'];
         $this->_projects['guids'][$name] = $guid;
      }
   }
   
   private function _load_people_by_current_project() {
      $current_project_guid = isset($this->_current_project_id) ? $this->_current_project_id : 0;
      $this->_load_participants_in_current_project($current_project_guid);
      $this->_load_people_available_but_not_on_current_project($current_project_guid);
   }
   
   private function _load_participants_in_current_project($current_project_guid) {
      $adapter = $this->_db_adapter;
      $query = "select * from person join participant" .
      "  on person.person_guid = participant.person_guid" .
      "     where participant.domain_guid = $current_project_guid" .
      "        order by person.name_given, person.name_last";
      $people_onboard = $adapter->db_get_all($query );
      $this->_persons_onboard = array();
      foreach($people_onboard as $person) {
         $this->_persons_onboard[] = $person;
         $this->_person_onboard_ids[] = $person['person_guid'];
      }
   }
   
   private function _load_people_available_but_not_on_current_project($current_project_guid) {
      $adapter = $this->_db_adapter;
      
      $query =
      "select person.* from person join person_visible on person.person_guid = person_visible.person_guid" .
         " left outer join" .
      " (select * from participant where participant.domain_guid = $current_project_guid)" .
         " as onboard" .
            " on person.person_guid = onboard.person_guid" .
               " where onboard.person_guid is null and" .
               " person_visible.project_space_guid = $this->_current_project_space_id" .
      "        order by person.name_given, person.name_last";
      $people_available = $adapter->db_get_all($query );
      $this->_persons = array();
      $this->_person_ids = array();
      foreach($people_available as $person) {
         $this->_persons[] = $person;
         $this->_person_ids[] = $person['person_guid'];
      }
   }
   
   private function _load_people_by_current_project_space() {
      $adapter = $this->_db_adapter;
      
      $query =
      "select person.* from person join person_visible on person.person_guid = person_visible.person_guid" .
               " where person_visible.project_space_guid = $this->_current_project_space_id" .
      "        order by person.name_given, person.name_last";
      $people_available = $adapter->db_get_all($query );
      foreach($people_available as $person) {
         $this->_persons[] = $person;
         $this->_person_ids[] = $person['person_guid'];
      }
   }
   
   public function get_current_project_space_id() {
      return $this->_current_project_space_id;
   }
   
   public function get_current_project_id() {
      return $this->_current_project_id;
   }
   
   public function get_current_project() {
      return $this->_current_project;
   }
   
   public function get_current_project_name() {
      if ( !isset($this->_current_project) ) {
         return '';
      }
      $current_project_name = $this->_current_project->get_project_name();
      return $current_project_name;
   }
   
   public function get_persons() {
      return $this->_persons;
   }
   
   public function get_person_onboard_ids() {
      return $this->_person_onboard_ids;
   }
   
   public function get_persons_onboard() {
      return $this->_persons_onboard;
   }
   
   public function get_persons_to_edit() {
      return $this->_persons_to_edit;
   }
   
   public function get_projects() {
      return $this->_projects;
   }
   
   public function get_tasks() {
      return $this->_tasks;
   }
}

include_once('user_interface.tmpl.php');
?>
