<?php
  /**
   * Name: ProjectSpace.php
   * Author: R. Joseph Newton
   * Class provided: ProjectSpace
   * Purpose: Provides a ProjectSpace class for use in project management
   *  applications. This class represents a partition within a Project
   *  Manager installation, within which Project objects and related
   *  objects can be developed and insulated from other objects managed
   *  by the installation.  Person objects are also visible only in the
   *  project spaces with which they are associated. although a given
   *  person may be associated with multiple project spaces.
   * Last modified: March 3, 2007
   */

/* ProjectSpace.php */

include_once('MySQLAdapterProjectManager.php');
include_once('Person.php');
include_once('Project.php');
include_once('utilities.php');

class ProjectSpace {
   private $_db_adapter;
   private $_project_space_guid;
   private $_name;
   private $_owner;
   private $_description;
   private $_person_ids;
   private $_persons;
   private $_modified;
   
   public function ProjectSpace($name, $owner, $description) {
      $this->_db_adapter = new MySQLAdapterProjectManager();
      $this->_name = $name;
      $this->_owner = $owner;
      $this->_description = $description;
      $this->_modified = true;
   }
   
   public function get_by_project_space_id($project_space_guid) {
      $adapter = new MySQLAdapterProjectManager();
      $project_space = NULL;
      $query = "select * from project_space where project_space_guid = $project_space_guid";
      $result = $adapter->db_get_first($query);
      $project_space = new ProjectSpace($result['name'], $result['owner'], $result['description']);
      $project_space->_project_space_guid = $project_space_guid;
      $project_space->_modified = false;
      return $project_space;
   }
   
   public function save() {
      if ( isset($this->_project_space_guid) ) {
         $result = $this->_update();
      } else {
         $result = $this->_create_new();
      }
      if ($result == 1) {
         $this->_modified = false;
      }
      return $result;
   }
   
   private function _update() {
      $adapter = $this->_db_adapter;
      $project_name = $adapter->db_escape($this->_name);
      $owner = $adapter->db_escape($this->_owner);
      $description = $adapter->db_escape($this->_description);
      $command = "update project_space set name = '$project_name'," .
      "owner = '$owner', description = '$description'";
      $command .= " where project_space_guid = $this->_project_space_guid";
      $result = $adapter->db_command($command);
      return $result;
   }
   
   private function _create_new() {
      $adapter = $this->_db_adapter;
      $guid = $adapter->create_table_guid('project_space');
      if (!$guid) {
         return false;
      }
      $this->_project_space_guid = $guid;
      $query = $this->_insert_project_space_query($guid);
      $result = $adapter->db_command($query);
      return $result;
   }
   
   private function _insert_project_space_query($guid) {
      $adapter = $this->_db_adapter;
      $project_name = $adapter->db_escape($this->_name);
      $owner = $adapter->db_escape($this->_owner);
      $description = $adapter->db_escape($this->_description);
      $query = "insert into project_space values " .
      " ($this->_project_space_guid, '$project_name', '$owner', '$description')";
      return $query;
   }
   
   public function add_project($name, $description, $lead, $co_lead = NULL,
    $start_date, $projected_finish = array(), $finish_date = array()) {
      $adapter = $this->_db_adapter;
      $guid = $this->_project_space_guid;
      $project = new Project($guid, $name, $description, $lead, $co_lead,
       $start_date, $projected_finish, $finish_date);
      $project->save();
      $project_id = $project->get_project_id();
      return $project;
   }
   
   public function add_task($parent_project, $name, $description, $lead, $co_lead = NULL,
    $start_date, $projected_finish = array(), $finish_date = array()) {
      print "Entered function, at least<br>\n";
      $adapter = $this->_db_adapter;
      $guid = $this->_project_space_guid;
      $task = new Task($parent_project, $guid, $name, $description, $lead, $co_lead,
       $start_date, $projected_finish, $finish_date);
      $task->save();
      $task_id = $task->get_project_id();
      return $task;
   }
   
   public function get_project_space_id() {
      return $this->_project_space_guid;
   }
}

?>
