<?php

/* ProjectSpacePrivilege.php */

/**
 * Name: ProjectSpacePrivilege.php [PHP class definition]
 * Author: R. Joseph Newton
 * Class Provided: ProjectSpacePrivilege
 * Purpose: Provides a ProjectSpacePrivilege class for use in
 *  project management applications.  Each object of this type
 *  represents a privilege held by one person or role in the scope
 *  of one Project Space.
 * Last Modified: March 29, 2007
 */

include_once('MySQLAdapterProjectManager.php');
include_once('Privilege.php');
include_once('ProjectSpace.php');

class ProjectSpacePrivilege {
   private $_privilege;
   private $_project_space_privilege_guid;
   private $_project_space_guid;
   private $_privilege_holder_guid;
   private $_is_role_bound;
   private $_include_cildren;
   private $_modified;
   
   public function ProjectSpacePrivilege($privilege_id, $project_space_id, $privilege_holder_id,
    $is_role_bound, $include_children) {
      $this->db_adapter = new MySQLAdapterProjectManager();
      
      $this->_privilege = Privilege::get_by_privilege_id($privilege_id);
      $this->_project_space_guid = $project_space_id;
      $this->_privilege_holder_guid = $privilege_holder_id;
      $this->_is_role_bound = $is_role_bound;
      $this->_include_children = $include_children;
      $this->_modified = true;
   }
   
   public function save() {
      if ( isset($this->privilege_guid) ) {
         $this->_update();
      } else {
         $this->_create_new();
      }
   }
   
   private function _update() {
      $adapter = $this->db_adapter;
      
      $is_role_bound = $this->_is_role_bound;
      $include_children = $this->_include_children;
      $command = 'update project_space_privilege' .
      " set is_role_bound = $is_role_bound, include_children = $include_children" .
      ' where privilege_guid = ' . $this->_privilege_guid;
      $result = $db_command($command);
      $this->_modified = false;
      return $result;
   }
   
   private function _create_new() {
      $adapter = $this->db_adapter;
      
      $psp_guid = $adapter->create_table_guid('project_space_privilege');
      $this->_project_space_privilege_guid = $psp_guid;
      
      $command = 'insert into project_space_privilege (project_space_privilege_guid,' .
      ' project_space_guid, holder_guid, privilege_guid, is_role_bound, include_children)' .
      " values ($psp_guid, $this->_project_space_guid, $this->_holder_guid," .
      " $this->_privilege_guid, $this->_is_role_bound, $this->_include_children)";
      $result = db_command($command);
      $this->_modified = false;
      return $result;
   }
   
   public function get_by_project_space_privilege_id($psp_id) {
      $adapter = new MySQLAdapterProjectManager();
      
      $query = 'select * from project_space_privilege' .
      " where project_space_privilege_guid = $psp_id";
      $result = db_get_first($query);
      if ( isset($result['error']) ) {
         return NULL;
      }
      $ps_id = $result['project_space_guid'];
      $holder_id = $result['holder_guid'];
      $privilege_id = $result['privilege_guid'];
      $is_role_bound = $result['is_role_bound'];
      $include_children = $result['include_children'];
      $ps_privilege = new ProjectSpacePrivilege($privilege_id, $ps_id, $holder_id,
      $is_role_bound, $include_children);
      $ps_privilege->_project_space_privilege_guid = $psp_id;
      return $guid;
   }
   
   public function get_privilege() {
      return $this->_privilege;
   }
   
   public function get_project_space_privilege_id() {
      return $this->_project_space_privilege_guid;
   }
   
   public function get_project_space_id() {
      return $this->_project_space_guid;
   }
   
   public function get_privilege_holder_id() {
      return $this->_privilege_holder_guid;
   }
   
   public function is_role_bound() {
      return $this->_is_role_bound;
   }
   
   public function includes_subtree() {
      return $this->_include_children;
   }
   
   public function include_subtree() {
      $this->_include_children = true;
      $this->modified = true;
   }
   
   public function exclude_subtree() {
      $this->_include_children = false;
      $this->modified = false;
   }
   
   public function get_privilege_name() {
      $name = $this->get_privilege->get_privilege_name();
      return $name;
   }
   
   public function get_description() {
      $name = $this->get_privilege()->get_description();
      return $name;
   }
}

?>

