<?php
/**
 * Name: utilities.php
 * Author: R. Joseph Newton
 * Purpose: Provides random utility functions
 & Last modified: February 2, 2007
 */

function webbify_text($text) {
   $webbified = preg_replace('/(\n|\r\n)/', "<br />$1", $text);
   return $webbified;
}

function date_ISO_to_array($str_date) {
   if ( !preg_match('/^\d{4}\-\d{2}\-\d{2}$/', $str_date) ) {
      return array();
   }
   list($year, $month, $day) = split('-', $str_date);
   $arr_date = array('year' => $year, 'month' => intval($month), 'day' => intval($day));
   return $arr_date;
}

function date_array_to_ISO(array $date = array()) {
   if ( !isset($date['year']) ) {
      return NULL;
   }
   $date_string = sprintf( "%04d-%02d-%02d", $date['year'], $date['month'], $date['day'] );
   return "'$date_string'";
}

function pretty_date(array $date) {
   $months = array(
      1  => 'January',
      2  => 'February',
      3  => 'March',
      4  => 'April',
      5  => 'May',
      6  => 'June',
      7  => 'July',
      8  => 'August',
      9  => 'September',
      10 => 'October',
      11 => 'November',
      12 => 'December');
   $month = $months[$date['month']];
   $day = $date['day'];
   $year = $date['year'];
   return "$month $day, $year";
}

function date_selector($prefix = '', array $preset_date = NULL, $widget_disabled = false) {
   $preset_month = isset($preset_date) ? $preset_date['month'] : NULL; 
   $preset_day = isset($preset_date) ? $preset_date['day'] : NULL; 
   $preset_year = isset($preset_date) ? $preset_date['year'] : NULL;
   $selector_string = <<<SELECTOR_OPEN

<!-- DATE SELECTOR SECTION  -- [apologies for skewed indentation--RJN] ----------------------------------->

         <div class="selector-date">

SELECTOR_OPEN;
   $selector_string .= month_select($prefix, $preset_month, $widget_disabled) . "\n" .
    day_select($prefix, $preset_day, $widget_disabled) . "\n" .
    year_select($prefix, $preset_year, $widget_disabled) . "\n";
   $selector_string .= <<<SELECTOR_CLOSE

         </div>
<!-- END DATE SELECTOR SECTION  --------------------------------------------------------------------------->

SELECTOR_CLOSE;
   return $selector_string;
}

function month_select($prefix, $preset_month = NULL, $widget_disabled = false) {
   $month_tag = $prefix ? $prefix . '-month' : 'month';
   $month_id = preg_replace('/.*\-/', '', $month_tag);
   $this_month = isset($preset_month) ? sprintf("%02d", $preset_month) : date('m');
   $disabled = $widget_disabled ? ' disabled' : '';
   $month_string = <<<SELECT_MONTH

         <div class="select-month" style="float: left">
            <select name="$month_tag" class="select-month" id="$month_tag"$disabled> 
               <option value="01"> January </option> <option value="02"> February </option> <option value="03"> March </option> 
               <option value="04"> April </option> <option value="05"> May </option> <option value="06"> June </option> 
               <option value="07"> July </option> <option value="08"> August </option> <option value="09"> September </option> 
               <option value="10"> October </option> <option value="11"> November </option> <option value="12"> December </option> 
            </select>
         </div>
SELECT_MONTH;
   $replacement_string = 'value="' . $this_month . '"';
   $month_string = preg_replace("/($replacement_string)>/", "$1 selected>", $month_string);
   return $month_string;
}

function day_select($prefix = '', $preset_day = NULL, $widget_disabled = false) {
   $day_tag = $prefix ? $prefix . '-day' : 'day';
   $day_id = preg_replace('/.*\-/', '', $day_tag);
   $this_day = isset($preset_day) ? sprintf("%02d", $preset_day) : date('d');
   $disabled = $widget_disabled ? ' disabled' : '';
   $day_string = <<<SELECT_DAY

         <div class="select-day" style="float: left">
            <select name="$day_tag" class="select-day" id="$day_tag"$disabled>
               <option value="01"> 1 </option> <option value="02"> 2 </option> <option value="03"> 3 </option> 
               <option value="04"> 4 </option> <option value="05"> 5 </option> <option value="06"> 6 </option> 
               <option value="07"> 7 </option> <option value="08"> 8 </option> <option value="09"> 9 </option> 
               <option value="10"> 10 </option> <option value="11"> 11 </option> <option value="12"> 12 </option> 
               <option value="13"> 13 </option> <option value="14"> 14 </option> <option value="15"> 15 </option> 
               <option value="16"> 16 </option> <option value="15"> 17 </option> <option value="18"> 18 </option> 
               <option value="19"> 19 </option> <option value="20"> 20 </option> <option value="21"> 21 </option> 
               <option value="22"> 22 </option> <option value="23"> 23 </option> <option value="24"> 24 </option> 
               <option value="25"> 25 </option> <option value="26"> 26 </option> <option value="27"> 27 </option> 
               <option value="28"> 28 </option> <option value="29"> 29 </option> <option value="30"> 30 </option>
               <option value="31"> 31 </option>
            </select>
         </div>
SELECT_DAY;
   $replacement_string = 'value="' . $this_day . '"';
   $day_string = preg_replace("/($replacement_string)>/", "$1 selected>", $day_string);
   return $day_string;
}

function year_select($prefix = '', $preset_year = NULL, $widget_disabled = false) {
   $year_tag = $prefix ? $prefix . '-year' : 'year';
   $year_id = preg_replace('/.*\-/', '', $year_tag);
   $this_year = isset($preset_year) ? $preset_year : date('Y');
   $disabled = $widget_disabled ? ' disabled' : '';
   $year_string = <<<SELECT_YEAR

         <div class="select-year" style="float: left">
            <select name="$year_tag" class="select-year" id="$year_tag"$disabled>
               <option value="1990"> 1990 </option> <option value="1991"> 1991 </option> <option value="1992"> 1992 </option> 
               <option value="1993"> 1993 </option> <option value="1994"> 1994 </option> <option value="1995"> 1995 </option> 
               <option value="1996"> 1996 </option> <option value="1997"> 1997 </option> <option value="1998"> 1998 </option> 
               <option value="1999"> 1999 </option> <option value="2000"> 2000 </option> <option value="2001"> 2001 </option> 
               <option value="2002"> 2002 </option> <option value="2003"> 2003 </option> <option value="2004"> 2004 </option> 
               <option value="2005"> 2005 </option> <option value="2006"> 2006 </option> <option value="2007"> 2007 </option> 
               <option value="2008"> 2008 </option> <option value="2009"> 2009 </option> <option value="2010"> 2010 </option> 
               <option value="2011"> 2011 </option> <option value="2012"> 2012 </option> <option value="2013"> 2013 </option> 
               <option value="2014"> 2014 </option> <option value="2015"> 2015 </option> <option value="2016"> 2016 </option> 
               <option value="2017"> 2017 </option> <option value="2018"> 2018 </option> <option value="2019"> 2019 </option> 
               <option value="2020"> 2020 </option> <option value="2021"> 2012 </option> <option value="2022"> 2022 </option> 
               <option value="2023"> 2023 </option> <option value="2024"> 2024 </option> <option value="2025"> 2025 </option> 
               <option value="2026"> 2026 </option> <option value="2027"> 2027 </option> <option value="2028"> 2028 </option> 
            </select>
         </div>
SELECT_YEAR;
   $replacement_string = 'value="' . $this_year . '"';
   $year_string = preg_replace("/($replacement_string)>/", "$1 selected>", $year_string);
   return $year_string;
}

function dump_array_to_page(array $array) {
   print "<br><br>\n<pre>\n";
   print_r($array);
   print "\n</pre>\n<br><br>";
}

function extract_from_POST_vars($base_string) {
   $result_array = array();
   $base_length = strlen($base_string);
   foreach ( array_keys($_POST) as $key ) {
      if (strpos($key, $base_string) === 0) {
         $short_key = substr($key, $base_length);
         $result_array[$short_key] = $_POST[$key];
      }
   }
   return $result_array;
}

?>
